/***
  help.c - simple help (what the buttons do) for CERN preprint xtiff
***/

#ifndef lint
static char sccs_id[] = "@(#) help.c 1.1@(#)";
#endif /* lint */
 

#include <stdio.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>
#define _GCC_WCHAR_T		/* emergency compile-me kludge */
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>

#include "endhelp.h"

#ifdef __STDC__
#define PP(args)    args
#else
#define PP(args)    ()
#endif

Boolean amhelpful PP((void));
void behelpful PP((void));
void beunhelpful PP((Widget w, XtPointer unused_1, XtPointer unused_2));
void makefbutton PP((Widget *wlist, Widget parent, int offset));
void makeflabel PP((Widget *wlist, Widget parent, int offset));

/*
  imported global data this module needs to know about
*/
extern Widget	shellWidget;
extern Pixmap	buttonpixmaps[];
extern Display *xDisplay;
extern int	xScreen;

/*
  the help text for each button.  the text is supposed to be displayed in 
  a fixed font (that's why it's all the same length).  there's no protection
  against the window opened for the help text being too small.  the user will
  have to resize it.
*/
static char	*button_blurb[] = {

"Quit the program, leave xtiff altogether  ",
"Go to & view the previous page in the     \
\ndocument, eg to page 1 from page 2",
"Go to & view the next page in the         \
\ndocument, eg to page 7 from page 6",
"Go to & view the very first page of the   \
\ndocument",
"Go to & view the very last page in the    \
\ndocument",
"Make the page smaller, fit more text into \
\nthe window",
"Make the page larger, make the text bigger",
"Display the page at full size (as scanned)",
"Shrink the document enough to fit it all  \
\ninto the window",
"Display the page unrotated (as scanned)   ",
"Display the page rotated to 90 degrees    \
\nclockwise, ie ^  becomes >",
"Display the page rotated to 180 degrees   \
\nclockwise, ie ^ becomes v",
"Display the page rotated to 270 degrees   \
\nclockwise, ie ^ becomes <",
"Select, go to & view an arbitrary page    \
\nin the document"

}; /* button_blurb */

static Widget	popupshell,
		form,
		wlist[42],
		exitbutton;
static	Boolean	poppedup = False;

Arg fargs[] = {
	{ XtNresizable, False }
};

Arg bargs[] = {
	{ XtNborderWidth, 0 }
};

Arg mbargs[] = { 
	{ XtNjustify, XtJustifyCenter }
};

Arg sargs[] = {
	{ XtNallowShellResize, False },
	{ XtNresizable, False }
};
	
Boolean
amhelpful()
{
 return poppedup;
} /* amhelpful */

void
makelabel(wlist, parent, offset)
Widget	*wlist,
	parent;
int	offset;
{ 
 Arg	args[3];
 char	str[15];

 (void)sprintf(str, "label%d", offset/3);
 wlist[offset] = XtCreateWidget(str, labelWidgetClass, parent, 
		bargs, XtNumber(bargs) ); 

 XtSetArg(args[0], XtNlabel, button_blurb[offset/3]);
 if( 32 > button_height )
	 XtSetArg(args[1], XtNheight, button_height + 8);
 else
	 XtSetArg(args[1], XtNheight, button_height);
 XtSetValues(wlist[offset], args, 2);

} /* makelabel */



void 
makebutton(wlist, parent, offset)
Widget	*wlist,
	parent;
int	offset;
{
 Arg	args[5];
 char	str[15];

 (void)sprintf(str, "button%d", offset/3);

 if( 1 == offset )
	 wlist[offset] = XtCreateWidget(str, commandWidgetClass, parent,
				mbargs, XtNumber(mbargs));
 else 
	 wlist[offset] = XtCreateWidget(str, labelWidgetClass, parent,
				mbargs, XtNumber(mbargs));
 
 XtSetArg(args[0], XtNbitmap, buttonpixmaps[offset/3]);

 XtSetValues(wlist[offset], args, 1);

 if( 1 == offset )
	XtAddCallback(wlist[offset], XtNcallback,  beunhelpful, NULL);
 else {
	XtSetArg(args[0], XtNsensitive, False);
	XtSetValues(wlist[offset], args, 1);
 } /* if !offset ... else */

} /* makebutton */

	

void
behelpful()
{
 int		i,
		maxlen = 42; 
 Arg		args[5];

 /*
   nothing to do if popped up
 */
 if( True == poppedup )
	return;

 popupshell = XtCreatePopupShell("What the buttons mean", 
			transientShellWidgetClass,
			shellWidget, sargs, XtNumber(sargs)); 

 XtSetArg(args[0], XtNwidth, (button_width + 20 + maxlen * 8) * 2);
 if( 32 < button_height )
	 XtSetArg(args[1], XtNheight, 
				(button_height + 5) * 8 + endhelp_height + 20);
 else
	 XtSetArg(args[1], XtNheight, 
				(button_height + 20) * 7 + endhelp_height + 15);
 XtSetArg(args[2], XtNallowShellResize, False);
 XtSetValues(popupshell, args, 3); 

 form = XtCreateManagedWidget("helpform", formWidgetClass,
			popupshell, fargs, XtNumber(fargs)); 

 for(i=0; i<42; i+=3) {
	 char str[20];
	
	 (void)sprintf(str, "box%d", i/3);
	 wlist[i] = XtCreateWidget(str, boxWidgetClass, form,
				bargs, XtNumber(bargs));

	 if( 32 > button_height )
		 XtSetArg(args[0], XtNheight, button_height + 8);
	 else
		 XtSetArg(args[0], XtNheight, button_height);

	 XtSetValues(wlist[i], args, 1);

	 if(i && !(i%3) ) {

		if( 21 != i ) {
			XtSetArg(args[0], XtNfromVert, wlist[i-3]);
			XtSetValues(wlist[i], args, 1);
		} /* if 21 */

		if( 21 <= i ) {
			XtSetArg(args[0], XtNfromHoriz, wlist[i-21]);
			XtSetValues(wlist[i], args, 1);
		} /* if 21 */

	 } /* if i */

	 makebutton(wlist, wlist[i], i+1);
	 makelabel(wlist, wlist[i], i+2);

	 XtManageChildren(&wlist[i+1], 2);
	 XtManageChild(wlist[i]);

 } /* for i */

 exitbutton = XtCreateWidget("endhelp", commandWidgetClass, form, bargs,
		XtNumber(bargs) );
 XtAddCallback(exitbutton, XtNcallback, beunhelpful, NULL);
 
 XtSetArg(args[0], XtNfromVert, wlist[18]);

 XtSetArg(args[1], XtNbitmap, XCreateBitmapFromData(xDisplay, 
						    RootWindow(xDisplay, 
								xScreen),
						    endhelp_bits, 
						    endhelp_width,
						    endhelp_height));
 XtSetArg(args[2], XtNjustify, XtJustifyCenter);
 XtSetArg(args[3], XtNheight, endhelp_height + 10);
 XtSetArg(args[4], XtNwidth, endhelp_width + 250 );
 XtSetValues(exitbutton, args, 5);
 XtManageChild(exitbutton);
 XtManageChild(popupshell);

 XtPopup(popupshell, XtGrabNone);
 poppedup = True;

} /* behelpful */


/* ARGSUSED */
void
beunhelpful(w, unused_1, unused_2)
Widget		w;
XtPointer	unused_1,
		unused_2;
{
 if( True != poppedup ) {
	(void)fprintf(stderr, 
		"xtiff: some problem with help system: popped up %d?",
								 poppedup);
	exit(1);
 } /* if True */

 XtPopdown(popupshell);
 XtDestroyWidget(popupshell); 

 poppedup = False;

} /* beunhelpful */


/*** help.c ***/

